"""
Simple VTK importer from an HDF5 file

Matthew Turk
matthewturk@gmail.com
"""

from enthought.tvtk.tools import ivtk
from enthought.tvtk.api import tvtk 
from enthought.traits.api import Float, HasTraits, Instance, Range, Any

#from yt.reason import *
import numpy as na
import tables

class IVTKScene(object):
    def __init__(self):
        window = ivtk.IVTKWithCrustAndBrowser(size=(800,600))
        window.open()
        self.window = window
        self.scene = window.scene

class ExtractedVTKHierarchicalDataSet(IVTKScene):

    _gid = 0 # AMRBoxes require unique ids
    _grid_boundaries_shown = False
    _grid_boundaries_actor = None

    def __init__(self, scene_frame = None, center = None):
        if scene_frame is None: scene_frame = IVTKScene()
        self.scene_frame = scene_frame
        self.field = "Density"
        self._take_log = True
        self._grids = []
        self._vtk_objs = []
        self.operators = []
        self._hdata_set = tvtk.HierarchicalBoxDataSet()
        self._min_val = 1e60
        self._max_val = -1e60
        self.center = na.array([0.13990784, 0.08222961, 0.17372131],
                               dtype='float64')
        self.left_edge = na.array([0.0,0.0,0.0], dtype='float64')
        self.right_edge = na.array([0.296875, 0.203125, 0.28125],
                               dtype='float64')
        self._file_handle = tables.openFile("extracted_data.h5")
        for n in self._file_handle.iterNodes("/"):
            self._add_extracted_grid(n)
        self._hdata_set.generate_visibility_arrays()
        self.scene_frame.scene.camera.focal_point = self.center

    def _add_extracted_grid(self, node):
        ug = tvtk.UniformGrid(origin=node.getAttr("origin"),
                    spacing=[node.getAttr("spacing")]*3,
                    dimensions=node.getAttr("dims"))
        scalars = node[:]
        ug.point_data.scalars = scalars.ravel()
        ug.point_data.scalars.name = self.field
        self._min_val = min(self._min_val, scalars.min())
        self._max_val = max(self._max_val, scalars.max())
        ab = tvtk.AMRBox()
        ab.set_lo_corner(node.getAttr("abox_left"))
        ab.set_hi_corner(node.getAttr("abox_right"))
        self._hdata_set.set_data_set(node.getAttr("Level"), node.getAttr("id"), ab, ug)
        self._hdata_set.set_refinement_ratio(node.getAttr("Level"),2)

    def toggle_grid_boundaries(self):
        if self._grid_boundaries_shown:
            self._grid_boundaries_actor.visibility = False
            return
        if self._grid_boundaries_actor is None:
            # We don't need to track this stuff right now.
            ocf = tvtk.OutlineCornerFilter(
                    executive=tvtk.CompositeDataPipeline(),
                    corner_factor = 0.5)
            ocf.input = self._hdata_set
            ocm = tvtk.HierarchicalPolyDataMapper(
                    input_connection = ocf.output_port)
            self._grid_boundaries_actor = tvtk.Actor(mapper = ocm)
            self.scene_frame.scene.add_actor(self._grid_boundaries_actor)
        else:
            self._grid_boundaries_actor.visibility = True

    def _add_plane(self, origin=(0.0,0.0,0.0), normal=(0,1,0)):
        plane = tvtk.Plane(origin=origin, normal=normal)
        cutter = tvtk.Cutter(executive = tvtk.CompositeDataPipeline(),
                             cut_function = plane)
        cutter.input = self._hdata_set
        clut = tvtk.LookupTable(hue_range=(0.0,1.00))
        clut.build()
        smap = tvtk.HierarchicalPolyDataMapper(
                        scalar_range=(self._min_val, self._max_val),
                        lookup_table=clut,
                        input_connection = cutter.output_port)
        sactor = tvtk.Actor(mapper=smap)
        self.scene_frame.scene.add_actors(sactor)
        return plane

    def add_plane(self, origin=(0.0,0.0,0.0), normal=(0,1,0)):
        self.operators.append(self._add_plane(origin, normal))
        return self.operators[-1]

    def add_x_plane(self):
        np = self._add_plane(self.center, normal=(1,0,0))
        self.operators.append(VTKCuttingPlane(
                vmin=self.left_edge[0], vmax=self.right_edge[0],
                vdefault = self.center[0],
                post_call = self.scene_frame.scene.render,
                plane = np, axis=0, coord=0.0))
        return self.operators[-1]

    def add_y_plane(self):
        np = self._add_plane(self.center, normal=(0,1,0))
        self.operators.append(VTKCuttingPlane(
                vmin=self.left_edge[1], vmax=self.right_edge[1],
                vdefault = self.center[1],
                post_call = self.scene_frame.scene.render,
                plane = np, axis=1, coord=0.0))
        return self.operators[-1]

    def add_z_plane(self):
        np = self._add_plane(self.center, normal=(0,0,1))
        self.operators.append(VTKCuttingPlane(
                vmin=self.left_edge[2], vmax=self.right_edge[2],
                vdefault = self.center[2],
                post_call = self.scene_frame.scene.render,
                plane = np, axis=2, coord=0.0))
        return self.operators[-1]

    def add_contour(self, val):
        cubes = tvtk.MarchingCubes(
                    executive = tvtk.CompositeDataPipeline())
        cubes.input = self._hdata_set
        cubes.set_value(0, val)
        cube_lut = tvtk.LookupTable(hue_range=(0.0,0.67))
        cube_lut.build()
        cube_mapper = tvtk.HierarchicalPolyDataMapper(
                                input_connection = cubes.output_port,
                                lookup_table=cube_lut)

        cube_actor = tvtk.Actor(mapper=cube_mapper)
        self.scene_frame.scene.add_actors(cube_actor)
        self.operators.append(MarchingCubes(cubes=cubes,
                    vmin=self._min_val, vmax=self._max_val,
                    vdefault=val,
                    post_call = self.scene_frame.scene.render))
        return self.operators[-1]

class VTKCuttingPlane(HasTraits):
    plane = Instance(tvtk.Plane)
    post_call = Any

    def __init__(self, vmin, vmax, vdefault, **traits):
        HasTraits.__init__(self, **traits)
        trait = Range(float(vmin), float(vmax), value=vdefault)
        self.add_trait("coord", trait)
        self.coord = vdefault

    def _coord_changed(self, old, new):
        orig = self.plane.origin[:]
        orig[self.axis] = new
        self.plane.origin = orig
        self.post_call()

class MarchingCubes(HasTraits):
    cubes = Instance(tvtk.MarchingCubes)
    post_call = Any

    def __init__(self, vmin, vmax, vdefault, **traits):
        HasTraits.__init__(self, **traits)
        trait = Range(float(vmin), float(vmax), value=vdefault)
        self.add_trait("value", trait)
        self.value = vdefault

    def _value_changed(self, old, new):
        self.cubes.set_value(0, new)
        self.post_call()

if __name__=="__main__":
    from enthought.pyface.api import GUI
    gui = GUI()
    ehds = ExtractedVTKHierarchicalDataSet()
    ehds.toggle_grid_boundaries()
    gui.start_event_loop()
