/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkAMRBox.h,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkAMRBox - represents a 3D uniform region in space
// .SECTION Description
// vtkAMRBox is similar to Chombo's Box. It represents a 3D
// region by storing indices for two corners (LoCorner, HiCorner).
// A few utility methods are provided.

#ifndef __vtkAMRBox_h
#define __vtkAMRBox_h

#include "vtkObject.h"
#include "vtkObjectFactory.h"

#include <assert.h>

template<int dimension>
struct vtkAMRBoxInitializeHelp;

template<int dimension>
void vtkAMRBoxInitialize(int *LoCorner, int *HiCorner, // member
                         const int *loCorner, const int *hiCorner, // local
                         vtkAMRBoxInitializeHelp<dimension>* = 0); // dummy parameter for vs6
// Helper to unroll the loop

class VTK_FILTERING_EXPORT vtkAMRBox : public vtkObject
{
public:
  // Public for quick access.
  
  // Description:
  // Cell position of the lower cell and higher cell defining the box.
  // Both cells are defined at the same level of refinement. The refinement
  // level itself is not specified.
  // Invariant: LoCorner<=HiCorner
  // Eg. LoCorner = {0,0,0}, HiCorner = {0,0,0} is an AMRBox with 1 cell
  int LoCorner[3];
  int HiCorner[3];

  // Description:
  // Default constructor: lower corner box made of one cell.
  vtkAMRBox();
  vtkTypeRevisionMacro(vtkAMRBox,vtkObject);
  
  // Description:
  // Constructor.
  // \pre dimensionality >= 2 && dimensionality <= 3
  vtkAMRBox(int dimensionality, const int* loCorner, const int* hiCorner);

  vtkAMRBox(const vtkAMRBox& other);
  static vtkAMRBox *New();
  void operator=(const vtkAMRBox& other);
  
  // Description:
  // Returns the number of cells (aka elements, zones etc.) in
  // the given region (for the specified refinement, see Coarsen()
  // and Refine() ).
  vtkIdType GetNumberOfCells();
  
  // Description:
  // Modify LoCorner and HiCorner by coarsening with the given
  // refinement ratio.
  // \pre valid_refinement: refinement>=2
  void Coarsen(int refinement);
  
  // Description:
  // Modify LoCorner and HiCorner by refining with the given
  // refinement ratio.
  // \pre valid_refinement: refinement>=2
  void Refine(int refinement);
  
  // Description:
  // Returns non-zero if the box contains the cell with
  // given indices.
  int DoesContainCell(int i, int j, int k);
  
  //BTX
  // Description:
  // Returns non-zero if the box contains `box`
  int DoesContainBox(vtkAMRBox const & box) const;
   //ETX
  
  // Description:
  // Print LoCorner and HiCorner
  void Print(ostream &os);
  
  // Description:
  // Check if cell position is HiCorner
  bool IsHiCorner(const int pos[3]) const;
  
  // Description:
  // Check if cell position is LoCorner
  bool IsLoCorner(const int pos[3]) const;

  void SetLoCorner(const int pos[3]);
  void SetHiCorner(const int pos[3]);
};

#endif
